//-----------------------------------------
// LScript Modeler template
//
// Copyright 2023 Michael Green
// mike@mikegreen.name
//

myversion=1.0;
@version 2.2
@warnings
@script modeler

// global values go here
c;
gemptys;
gfglyrs;
gbglyrs;
gbglyr;
gtotal;
surflist;
gbbox;

main
{
    // requester code goes here
    gemptys=lyrempty();
    gfglyrs=lyrfg();
    gbglyrs=lyrbg();
    if (gbglyrs)
			gbglyr=gbglyrs[1];
		if(gemptys.contains(gbglyr))
			gbglyr=nil;

		selmode(DIRECT);
		gbbox=boundingbox();
		editbegin();
		mypoints=points;
		gtotal=<0,0,0>;
		if(mypoints)
		{
			for(i=1;i<=mypoints.size();i++)
			{
        	gtotal=gtotal+pointinfo(mypoints[i]);
			}
			gtotal=gtotal/mypoints.size();
		}                                                              // Find the average center of the selected points to use as the center of the dial.
		editend();

		fontlist[1]="Temp";
    for(i=1;i<=fontcount();i++)
    {
        fontlist[i]=fontname(i);
    }
	myobj=Mesh(0);
    surflist[1]="Dial";
    surflist=Surface(myobj);
    if(surflist==nil)
    	surflist[1]="Dial";


    myset=@ 0.0, 1.0, 12, 0, 360, 0.5, 1, 0.03, 5, 0.05, 0.05, 0.01, 1, "Digits","Ticks", 3, 0.0 @;

		for(i=1;i<=myset.size();i++)
		{
			myset[i]=recall("MG_Dial_Creator_"+i.asStr(),myset[i]);
		}
		if(myset[7]>fontlist.size())
			myset[7]==1;

		CreateDial(myset,gemptys[1],gfglyrs[1],gtotal,gbglyr);

    reqbegin("MG Dial Creator V"+myversion);
//debug();

    c[1] = ctlnumber("Start Value",myset[1]);
    c[2] = ctlnumber("Increment",myset[2]);
    c[3] = ctlinteger("Total Digits",myset[3]);
    c[4] = ctlangle("Start Angle",myset[4]);
    c[5] = ctlangle("End Angle",myset[5]);
    c[6] = ctlpercent("Radius",myset[6]);
	c[7] = ctlpopup("Font",myset[7],fontlist);
	c[8] = ctlpercent("Font size%",myset[8]);
	c[9] = ctlinteger("Ticks per increment", myset[9]);
	c[10] = ctlpercent("Tick inset%",myset[10]);
	c[11] = ctldistance("Tick length",myset[11]);
	c[12] = ctldistance("Tick width",myset[12]);
	c[13] = ctlchoice("Numeral Angle",myset[13],@"Angled","Upright"@);
	c[14] = ctlsurface("Digit Surface",myset[14]);
	d[1] = ctlmenu("Digit Surface",surflist,"ChangeDigitSurf");
	c[15] = ctlsurface("Tick Surface",myset[15]);
	d[1] = ctlmenu("Tick Surface",surflist,"ChangeTickSurf");
	c[16] = ctlchoice("Direction",myset[16],@"X","Y","Z"@);
	c[17] = ctlangle("Pitch",myset[17]);

	d[2] = ctlbutton("Automatic size",100,"AutoSize");
//	d[3] = ctlbutton("FrameSelected",100,"FrameSelected");
	h = ctlpopup("Help",1,	@"This plugin creates a dial with digits and ticks. If you",
							"have one or more points selected when the plugin is",
							"activated it will use the average of them as the centre.",
							"Tick inset is a percentage of the radius, 100% will place the",
							"ticks on the radius, through the numerals which sit on the radius.",
							"Automatic size will size the dial to the smallest dimension",
							"in the chosen direction, unless that value is zero, when it will",
							"set the Radius to 1m.",
							"Pitch will tilt the Numerals and Ticks if that's needed.",
							"If you have a background layer that will be used as the tick."  @);

	ctlrefresh(c[1],"Refresh");
	ctlrefresh(c[2],"Refresh");
	ctlrefresh(c[3],"Refresh");
	ctlrefresh(c[4],"Refresh");
	ctlrefresh(c[5],"Refresh");
	ctlrefresh(c[6],"Refresh");
	ctlrefresh(c[7],"Refresh");
	ctlrefresh(c[8],"Refresh");
	ctlrefresh(c[9],"Refresh");
	ctlrefresh(c[10],"Refresh");
	ctlrefresh(c[11],"Refresh");
	ctlrefresh(c[12],"Refresh");
	ctlrefresh(c[13],"Refresh");
	ctlrefresh(c[14],"Refresh");
	ctlrefresh(c[15],"Refresh");
	ctlrefresh(c[16],"Refresh");
	ctlrefresh(c[17],"Refresh");

	if (!reqpost())
	{
		undo();
		if(mypoints)
		{
			selmode(USER);
			selpoint(SET,POINTID,mypoints);
		}

		return;
	}

    // get requester control values here
	for (i=1;i<=c.size();i++)
	{
		myset[i]=getvalue(c[i]);
		store("MG_Dial_Creator_"+i.asStr(),myset[i]);
 	}
    reqend();

    // processing code goes here
}

Refresh:value
{
	for (i=1;i<=c.size();i++)
	{
		myset[i]=getvalue(c[i]);
		store("MG_Dial_Creator_"+i.asStr(),myset[i]);
 	}

 	if(myset[3]<2)
	{
		myset[3]=2;
		setvalue(c[3],2);           //Make total non-negative
		return;
	}

	undo();
	CreateDial(myset,gemptys[1],gfglyrs[1],gtotal,gbglyr);
}

ChangeDigitSurf:value
{
		setvalue(c[14],surflist[value]);
}

ChangeTickSurf:value
{
		setvalue(c[15],surflist[value]);
}

AutoSize
{
	mydimension=gbbox[2]-gbbox[1];
	mydir=getvalue(c[16]);
	if(mydir==1)
		myradius=(min(mydimension.z,mydimension.y))/2;
	if(mydir==2)
		myradius=(min(mydimension.x,mydimension.z))/2;
	if(mydir==3)
		myradius=(min(mydimension.x,mydimension.y))/2;
	if(myradius==0)
		myradius=1;
	setvalue(c[6],myradius);
}

FrameSelected
{
	CommandInput("autoscale");
}

CreateDial:pset,pemptylyr,pfglyr,pcentre,pbglyr
{

	undogroupbegin();
	selmode(USER);
	lyrsetfg(pemptylyr);
	myinitv=pset[1];
	myincre=pset[2];
	mytotal=pset[3];
	mystang=pset[4];
	myenang=pset[5];
	myradiu=pset[6];
	myfontf=pset[7];
	myfonts=pset[8];
	myticks=pset[9];
	mytkins=pset[10];
	mytklen=pset[11];
	mytkwid=pset[12];
	mynangl=pset[13];
	mysurfa=pset[14];
	mysurft=pset[15];
	mydirec=pset[16];
	mypitch=pset[17];

	mycurrentdigit=myinitv;
	myangleincrement=(myenang - mystang)/(mytotal - 1);

	for (i=1;i<=mytotal;i++)                                         								// Do the numerals
	{
		maketext(mycurrentdigit.asStr(),myfontf,SHARP,0,myfonts*myradiu,Z);
		mybbox=boundingbox();
		lower=mybbox[1];
		upper=mybbox[2];
		move(0-((upper.x+lower.x)/2),0-((upper.y+lower.y)/2),0);   										// centering numerals
		rotate(mypitch,X); 																														// Rotating by pitch
		rotate((mystang+(i - 1)*myangleincrement)*-1*(mynangl - 1),Z);              	// rotate if upright
		move(0,myradiu,0);                                        										// Moves out
		rotate(mystang+(i - 1)*myangleincrement,Z);               										// Rotates to position
		changesurface(mysurfa);
		selhide();                                                										// Hides for the next digit
		mycurrentdigit=mycurrentdigit+myincre;
 	}

	lowcorner=<0 - mytkwid/2, 0 - mytklen/2,0>;                     								//tick dimensions
	highcorner=<mytkwid/2,mytklen/2,0>;

	myend=0;                                           															// Check to add end tick
//	if(mymod)
		myend++;
	if (pbglyr)
	{
		lyrsetfg(pbglyrs);
		copy();
		lyrsetfg(pemptylyr);
	}
	else
	{
		makebox(lowcorner,highcorner);
		flip();                                           						//Otherwise it's backwards.
		cut();
	}
	if(myticks!=0)
	{
		mytickinc=myangleincrement/myticks;
		for (i=1;i<=(mytotal - 1)*myticks+myend;i++)                              // do the ticks
		{
			paste();
			move(0, myradiu*mytkins-myradiu ,0);                                  // Moves to relative position
			rotate(mypitch,X);                                                    // Rotates by pitch
			move(0, myradiu ,0);                                  // Moves to position
			rotate(mystang+(i - 1)*mytickinc,Z);               	// Rotates to position
			changesurface(mysurft);
			selhide();
		}
 	}
	selunhide();
	if(mydirec==1)
		rotate(90,Y);
	if(mydirec==2)
		rotate(90,X);
	move(pcentre);
	changepart("Dial");
	cut();
	lyrsetfg(pfglyr);
	paste();
	undogroupend();

}
